const mongoose = require('mongoose');
const jwt = require('jsonwebtoken');
const config = require('../../config/config');
const Joi = require('joi');

// Define the schema for the messages
const messageSchema = new mongoose.Schema({
  message: {
    type: String,
    required: true,
  },
  date: {
    type: Date,
    required: true,
    default: Date.now,
  },
  status: {
    type: String,
    enum: ['Active', 'Expired'],
    default: 'Active',
  },
});

const employeeSchema = new mongoose.Schema({
  name: { type: String, required: true },
  email: { type: String, required: true, unique: true },
  password: { type: String },
  refreshToken: { type: String },
  employeeId: { type: String, unique: true },
  type: { type: String },
  designation: { type: String },
  skillset: [{ type: mongoose.Schema.Types.ObjectId, ref: 'Skillset' }],
  status: { type: String },
  billableStatus: { type: String },
  dateOfJoining: { type: String },
  workType: { type: String },
  role: { type: mongoose.Schema.Types.ObjectId, required: true, ref: 'Role' },
  profilePicture: { type: String },
  mustChangePassword: { type: Boolean, default: true },
  TAMofProjects: [{ type: mongoose.Schema.Types.ObjectId, ref: 'Project' }],
  terminationDate:{ type: Date },
  Projects: [
    {
      projectId: { type: mongoose.Schema.Types.ObjectId, ref: 'ProjectTeamMember' },
      benchStatus: { type: String, required: true },
      active:{ type: Boolean, default: true },
    }
  ],
  visibility: { type: Boolean, default: true },

  // Add the messages array to store message notifications for each employee
  messages: [messageSchema],

  // Timer field for tracking work timers for employees
  timer: [
    {
      isRunning: { type: Boolean, default: false },
      project: { type: mongoose.Schema.Types.ObjectId, ref: 'Project' }, // Reference to a project
      startTime: { type: Date, default: null }, // When the timer was started
      Category: { type: String, default: '' }, // A brief summary of the task being tracked
      task: { type: String, default: '' }, // The task description
    }
  ],

}, { timestamps: true });

employeeSchema.methods.generateAuthToken = function () {
  const secret = config.ACCESS_TOKEN_SECRET || 'fallback_secret_key';
  return jwt.sign({ _id: this._id }, secret, { expiresIn: '15m' });
};

employeeSchema.methods.generateRefreshToken = function () {
  const secret = config.REFRESH_TOKEN_SECRET || 'fallback_refresh_secret_key';
  return jwt.sign({ _id: this._id }, secret, { expiresIn: '7d' });
};

employeeSchema.statics.validateEmployee = (employee) => {
  const schema = Joi.object({
    name: Joi.string().required(),
    email: Joi.string().email().required(),
    password: Joi.string().min(6).allow(null),
    employeeId: Joi.string().allow(null),
    type: Joi.string().allow(null),
    designation: Joi.string().allow(null),
    skillset: Joi.array().items(Joi.string().regex(/^[0-9a-fA-F]{24}$/)),
    billableStatus: Joi.string(),
    dateOfJoining: Joi.date(),
    workType: Joi.string().allow(null),
    role: Joi.string().regex(/^[0-9a-fA-F]{24}$/),
    mustChangePassword: Joi.boolean(),
    terminationDate:Joi.date(),
  });
  return schema.validate(employee);
};

employeeSchema.methods.calculateTotalHoursAllocated = async function () {
  const employee = this;

  // Fetch all projects for this employee, with populated project details
  await employee.populate({
    path: 'Projects.projectId',
    populate: { path: 'project', select: '_id name' }  // Populate the actual project details
  }).execPopulate();

  const projectHoursMap = new Map();  // To keep track of unique projects and their hours

  // Loop through the projects and sum up hours based on the conditions
  employee.Projects.forEach(project => {
    const projectId = project.projectId.project._id.toString();  // Get actual project ID
    if (project.benchStatus === 'Active') {  // Only include active projects
      if (!projectHoursMap.has(projectId)) {
        projectHoursMap.set(projectId, project.projectId.hoursAllocated);
      }
    }
  });

  // Calculate the total hours allocated
  const totalHours = Array.from(projectHoursMap.values()).reduce((acc, hours) => acc + hours, 0);

  // Update the totalHoursAllocated field
  employee.totalHoursAllocated = totalHours;

  // Save the employee document with updated totalHoursAllocated
  await employee.save();

  return totalHours;
};

const Employee = mongoose.model('Employee', employeeSchema);

module.exports = Employee;
