const mongoose = require('mongoose');
const { Schema } = mongoose;
const Joi = require('joi');
const JoiObjectId = require('joi-objectid')(Joi);

const milestoneSchema = new Schema({
  project: { type: Schema.Types.ObjectId, ref: 'Project', required: true },
  name: { type: String, required: true },
  startDate: { type: Date, required: true },
  endDate: { type: Date, required: true },
  estimatedHours: { type: Number, required: true },
  releaseDate: { type: Date, required: true },
  status: { type: String, required: true, enum: ['Not Started', 'In Review', 'Ongoing', 'Completed UAT', 'Closed'], default: 'Not Started' },
  visibility: { type: Boolean, default: true }
});

// Compound index to ensure milestone name is unique per project
milestoneSchema.index({ project: 1, name: 1 }, { unique: true });

const Milestone = mongoose.model('Milestone', milestoneSchema);

// Function to ensure indexes
const ensureIndexes = async () => {
  try {
    await Milestone.collection.dropIndexes(); // Remove all existing indexes
    await Milestone.createIndexes(); // Create new indexes
  } catch (error) {
    console.error('Error recreating Milestone indexes:', error);
  }
};

// Call this function when your application starts
ensureIndexes();

// Validation schema for Joi
const validateMilestone = (milestone) => {
  const schema = Joi.object({
    project: JoiObjectId().required(),
    name: Joi.string().required(),
    startDate: Joi.date().required(),
    endDate: Joi.date().required(),
    estimatedHours: Joi.number().required(),
    releaseDate: Joi.date().required(),
    status: Joi.string().valid('Not Started', 'In Review', 'Ongoing', 'Completed UAT', 'Closed').required(),
    visibility: Joi.boolean(),
    _id: JoiObjectId().optional()
  });
  return schema.validate(milestone);
};

module.exports = { Milestone, validateMilestone };
