const teamRepository = require('../team.repository');
const Employee = require('../../../database/models/employeeSchema');
const bcrypt = require('bcrypt');
const config = require('../../../config/config');
const ProjectTeamMember = require('../../../database/models/project/projectTeamMemberSchema');

const getTeam = async (
  search,
  page = 1,
  limit = 25,
  fetchAll = false,
  filter = 'Current'
) => {
  return await teamRepository.findAllWithSearchAndPagination(
    search,
    page,
    limit,
    fetchAll,
    filter
  );
};

const getTeamMemberById = async (id) => {
  return await teamRepository.findById(id);
};

const addTeamMember = async (teamMemberData) => {
  const { password, ...otherData } = teamMemberData;

  // Hash the password
  const saltRounds = Number(config.SALT) || 10;
  const hashedPassword = await bcrypt.hash(password, saltRounds);

  // Create the new team member with hashed password
  const newTeamMember = await teamRepository.create({
    ...otherData,
    password: hashedPassword,
    mustChangePassword: true,
  });

  // Remove sensitive data before returning
  const teamMemberToReturn = newTeamMember.toObject();
  delete teamMemberToReturn.password;
  delete teamMemberToReturn.refreshToken;

  return teamMemberToReturn;
};

const editTeamMember = async (id, teamMemberData) => {
  const employee = await teamRepository.findById(id);
  if (!employee) {
    throw new Error('Employee not found');
  }

  // Directly update the employee with the provided data
  return await teamRepository.updateById(id, teamMemberData);
};

const editProjectBenchStatus = async (
  employeeId,
  projectId,
  benchStatus,
  active
) => {
  const employee = await teamRepository.findById(employeeId);
  if (!employee) {
    throw new Error('Employee not found');
  }

  const projectIndex = employee.Projects.findIndex(
    (project) => project.projectId._id.toString() === projectId
  );
  if (projectIndex === -1) {
    throw new Error('Project not found for this employee');
  }

  // Update the benchStatus for the specific project
  employee.Projects[projectIndex].benchStatus = benchStatus;
  employee.Projects[projectIndex].active = active;

  return await teamRepository.updateById(employeeId, {
    Projects: employee.Projects,
  });
};

const deleteTeamMember = async (id, terminationDate) => {
  // eslint-disable-next-line no-useless-catch
  try {
    const employee = await Employee.findById(id);
    if (!employee) {
      throw new Error('Team member not found');
    }

    // Update ProjectTeamMember documents
    const projectUpdates = employee.Projects.map(async (project) => {
      await ProjectTeamMember.findByIdAndUpdate(project.projectId, {
        $set: { visibility: false },
      });
    });

    // Wait for all project updates to complete
    await Promise.all(projectUpdates);

    // Update employee's projects
    employee.Projects = employee.Projects.map((project) => ({
      ...project,
      benchStatus: 'On bench',
      active: false,
    }));

    // Clear the TAMofProjects array
    employee.TAMofProjects = [];


    // Update employee visibility, status, and set termination date
    employee.visibility = false;
    employee.status = 'Inactive';
    employee.terminationDate = new Date(terminationDate);

    // Save the updated employee
    await employee.save();

    return employee;
  } catch (error) {
    throw error;
  }
};

const getEmployeeStats = async () => {
  const [totalCurrentEmployees] =
    await teamRepository.getTotalCurrentEmployees();
  const [totalResignedEmployees] =
    await teamRepository.getTotalResignedEmployees();
  const [totalAllEmployees] = await teamRepository.getTotalAllEmployees();
  const [totalBillableEmployees] =
    await teamRepository.getTotalBillableEmployees();
  const [totalNonBillableEmployees] =
    await teamRepository.getTotalNonBillableEmployees();
  const [totalRemoteEmployees] = await teamRepository.getTotalRemoteEmployees();
  const [totalResourcesOnBench] =
    await teamRepository.getTotalResourceOnBench();

  return {
    totalCurrentEmployees: totalCurrentEmployees
      ? totalCurrentEmployees.totalCurrentEmployees
      : 0,
    totalResignedEmployees: totalResignedEmployees
      ? totalResignedEmployees.totalResignedEmployees
      : 0,
    totalAllEmployees: totalAllEmployees
      ? totalAllEmployees.totalAllEmployees
      : 0,
    totalBillableEmployees: totalBillableEmployees
      ? totalBillableEmployees.totalBillableEmployees
      : 0,
    totalNonBillableEmployees: totalNonBillableEmployees
      ? totalNonBillableEmployees.totalNonBillableEmployees
      : 0,
    totalRemoteEmployees: totalRemoteEmployees
      ? totalRemoteEmployees.totalRemoteEmployees
      : 0,
    totalResourcesOnBench: totalResourcesOnBench
      ? totalResourcesOnBench.totalResourcesOnBench
      : 0,
  };
};

const addProjectToEmployee = async (
  employeeId,
  projectId,
  benchStatus,
  active
) => {
  const employee = await teamRepository.findById(employeeId);
  if (!employee) {
    throw new Error('Employee not found');
  }

  // Push a new project to the Projects array
  employee.Projects.push({ projectId, benchStatus, active });

  // Save the updated employee document
  return await teamRepository.updateById(employeeId, {
    Projects: employee.Projects,
  });
};


const updateEmployeeMessageStatus = async (employeeId, date, status) => {
  try {
    const updatedEmployee = await teamRepository.updateEmployeeMessageStatus(employeeId, date, status);

    if (!updatedEmployee) {
      return null;
    }

    return updatedEmployee;
  } catch (error) {
    console.error('Error updating employee message status:', error);
    throw error;
  }
};




module.exports = {
  getTeam,
  getTeamMemberById,
  addTeamMember,
  editTeamMember,
  editProjectBenchStatus,
  deleteTeamMember,
  getEmployeeStats,
  addProjectToEmployee,
  updateEmployeeMessageStatus
};
