const teamService = require('./Services/team.service');
const sendEmail = require('../../utils/sendEmail2');

const getTeam = async (req, res) => {
  try {
    const { search, page = 1, limit = 25, fetchAll = false, filter = 'Current' } = req.query;
    const parsedPage = parseInt(page, 10);
    const parsedLimit = parseInt(limit, 10);
    
    if (isNaN(parsedPage) || isNaN(parsedLimit) || parsedPage < 1 || parsedLimit < 1) {
      return res.status(400).json({ error: 'Invalid page or limit parameters' });
    }
    
    const team = await teamService.getTeam(search, parsedPage, parsedLimit, fetchAll === 'true', filter);
    res.json(team);
  } catch (error) {
    console.error(error);
    res.status(500).send('Server error');
  }
};


const getTeamMemberById = async (req, res) => {
  try {
    const id = req.params.id;
    if (!id) {
      return res.status(400).send('ID is required');
    }

    const teamMember = await teamService.getTeamMemberById(id);
    if (!teamMember) {
      return res.status(404).send('Team member not found');
    }

    res.json(teamMember);
  } catch (error) {
    console.error('Error getting team member by ID:', error);
    res.status(500).send('Server error');
  }
};


const addTeamMember = async (req, res) => {
  try {
    const { email, password, sendEmailToUser, ...otherData } = req.body;
    
    if (!email || !password) {
      return res.status(400).send('Email and password are required');
    }

    const newTeamMember = await teamService.addTeamMember({ email, password, ...otherData });

    if (sendEmailToUser) {
      const subject = 'Your Account Registration';
      const text = `${password}`;
      await sendEmail(email, subject, text);
    }

    res.json({
      message: 'Team member added successfully',
      teamMember: newTeamMember
    });
  } catch (error) {
    if (error.code === 11000) {
      console.error(error);
      res.status(400).send({ message: 'Duplicate key error', details: error.keyValue });
    } else {
      res.status(500).send({ message: 'Internal server error', details: error.message });
    }
  }
};


const editTeamMember = async (req, res) => {
  try {
    const id = req.params.id; 
    if (!id) {
      return res.status(400).send('ID is required');
    }
        
    const updatedTeamMember = await teamService.editTeamMember(id, req.body);
    res.json(updatedTeamMember);
  } catch (error) {
    console.error('Error editing team member:', error);
    res.status(500).send('Server error');
  }
};

const editProjectBenchStatus = async (req, res) => {
  try {
    const { employeeId, projectId } = req.params;
    const { benchStatus,active } = req.body; 

    if (!employeeId || !projectId) {
      return res.status(400).send('Employee ID and Project ID are required');
    }

    const updatedTeamMember = await teamService.editProjectBenchStatus(employeeId, projectId, benchStatus,active);
      
    res.json(updatedTeamMember);
  } catch (error) {
    console.error('Error updating project bench status:', error);
    res.status(500).send('Server error');
  }
};

const deleteTeamMember = async (req, res) => {
  try {
    const id = req.params.id;
    const { terminationDate } = req.body;

    if (!id) {
      return res.status(400).send('ID is required');
    }
    if (!terminationDate) {
      return res.status(400).send('Termination date is required');
    }

    const deletedMember = await teamService.deleteTeamMember(id, terminationDate);
    res.json({ message: 'Team member status updated and termination date set', deletedMember });
  } catch (error) {
    if (error.message === 'Team member not found') {
      res.status(404).json({ message: error.message });
    } else {
      console.error(error);
      res.status(500).json({ message: 'Server error', error: error.message });
    }
  }
};


const getEmployeeStats = async (req, res) => {
  try {
    const stats = await teamService.getEmployeeStats();
    res.json(stats);
  } catch (error) {
    console.error('Error getting employee stats:', error);
    res.status(500).send('Server error');
  }
};

const addProjectToEmployee = async (req, res) => {
  try {
    const { employeeId } = req.params;
    const { projectId, benchStatus, active } = req.body;

    if (!employeeId || !projectId) {
      return res.status(400).send('Employee ID and Project ID are required');
    }

    const updatedEmployee = await teamService.addProjectToEmployee(employeeId, projectId, benchStatus, active);
    res.json(updatedEmployee);
  } catch (error) {
    console.error('Error adding project to employee:', error);
    res.status(500).send('Server error');
  }
};

const updateEmployeeMessageStatus = async (req, res) => {
  try {
    const { employeeId } = req.params;
    const { date, status } = req.body;

    console.log('Received date:', date); // Debug log

    if (!date || !status) {
      return res.status(400).json({ error: 'Date and status are required' });
    }

    // Validate the date
    const parsedDate = new Date(date);
    if (isNaN(parsedDate.getTime())) {
      return res.status(400).json({ error: 'Invalid date format. Please use YYYY-MM-DD or a valid ISO 8601 date string.' });
    }

    console.log('Parsed date:', parsedDate.toISOString()); // Debug log

    const updatedEmployee = await teamService.updateEmployeeMessageStatus(employeeId, parsedDate.toISOString(), status);

    if (!updatedEmployee) {
      return res.status(404).json({ error: 'Employee or message not found' });
    }

    // Log the updated message for debugging
    const updatedMessage = updatedEmployee.messages.find(msg => 
      new Date(msg.date).toDateString() === parsedDate.toDateString() && msg.status === status
    );
    console.log('Updated message:', updatedMessage);

    res.json({ message: 'Message status updated successfully', employee: updatedEmployee });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: 'Server error' });
  }
};


module.exports = {
  getTeam,
  getTeamMemberById,
  addTeamMember,
  editTeamMember,
  editProjectBenchStatus,
  deleteTeamMember,
  getEmployeeStats,
  addProjectToEmployee,
  updateEmployeeMessageStatus 
};