const Employee = require('../../database/models/employeeSchema');
const fs = require('fs').promises;
const path = require('path');
//const applySearchAndPagination = require('../../utils/paginationAndSearch');


const findAllWithSearchAndPagination = async (search, page, limit, fetchAll = false, filter = 'Current') => {
  // Ensure page and limit are numbers
  const pageNum = Math.max(1, parseInt(page, 10));
  const limitNum = Math.max(1, parseInt(limit, 10));

  const aggregationPipeline = [];
  
  if (filter === 'Current') {
    aggregationPipeline.push({ $match: { visibility: true } });
  } else if (filter === 'Resigned') {
    aggregationPipeline.push({ $match: { visibility: false } });
  }

  // Lookups for referenced fields
  aggregationPipeline.push(
    { $lookup: {
      from: 'skillsets',
      localField: 'skillset',
      foreignField: '_id',
      as: 'skillset'
    }},
    { $lookup: {
      from: 'roles',
      localField: 'role',
      foreignField: '_id',
      as: 'role'
    }},
    { $lookup: {
      from: 'projectteammembers',
      localField: 'Projects.projectId',
      foreignField: '_id',
      as: 'projectTeamMembers'
    }}
  );

  // Unwind arrays (but preserve null values)
  aggregationPipeline.push(
    { $unwind: { path: '$skillset', preserveNullAndEmptyArrays: true } },
    { $unwind: { path: '$role', preserveNullAndEmptyArrays: true } }
  );

  // Apply search query if provided
  if (search) {
    aggregationPipeline.push({
      $match: {
        $or: [
          { name: { $regex: search, $options: 'i' } },
          { designation: { $regex: search, $options: 'i' } },
          { type: { $regex: search, $options: 'i' } },
          { employeeId: { $regex: search, $options: 'i' } },
          { 'skillset.name': { $regex: search, $options: 'i' } }
        ]
      }
    });
  }

  // Group to remove duplicates caused by unwinding
  aggregationPipeline.push({
    $group: {
      _id: '$_id',
      name: { $first: '$name' },
      email: { $first: '$email' },
      employeeId: { $first: '$employeeId' },
      type: { $first: '$type' },
      designation: { $first: '$designation' },
      skillset: { $addToSet: '$skillset' },
      status: { $first: '$status' },
      billableStatus: { $first: '$billableStatus' },
      dateOfJoining: { $first: '$dateOfJoining' },
      workType: { $first: '$workType' },
      role: { $first: '$role' },
      profilePicture: { $first: '$profilePicture' },
      Projects: { $first: '$Projects' },
      projectTeamMembers: { $first: '$projectTeamMembers' },
      terminationDate: { $first: '$terminationDate' },
      visibility: { $first: '$visibility' }
    }
  });

  // Lookup for project details
  aggregationPipeline.push({
    $lookup: {
      from: 'projects',
      localField: 'projectTeamMembers.project',
      foreignField: '_id',
      as: 'projectDetails'
    }
  });

  // Prepare projects array
  aggregationPipeline.push({
    $addFields: {
      Projects: {
        $map: {
          input: '$Projects',
          as: 'project',
          in: {
            projectId: {
              $mergeObjects: [
                { $arrayElemAt: ['$projectTeamMembers', { $indexOfArray: ['$projectTeamMembers._id', '$$project.projectId'] }] },
                { 
                  project: { 
                    $arrayElemAt: [
                      '$projectDetails', 
                      { $indexOfArray: ['$projectDetails._id', { $arrayElemAt: ['$projectTeamMembers.project', { $indexOfArray: ['$projectTeamMembers._id', '$$project.projectId'] }] }] }
                    ]
                  }
                }
              ]
            },
            benchStatus: '$$project.benchStatus',
            active: '$$project.active'
          }
        }
      }
    }
  });

  // Remove temporary fields
  aggregationPipeline.push({
    $project: {
      projectTeamMembers: 0,
      projectDetails: 0
    }
  });

  // Sort by employee name
  aggregationPipeline.push({ $sort: { name: 1 } });

  // Apply pagination if not fetching all
  if (!fetchAll) {
    aggregationPipeline.push(
      { $skip: (pageNum - 1) * limitNum },
      { $limit: limitNum }
    );
  }

  // Execute the aggregation
  try {
    const results = await Employee.aggregate(aggregationPipeline);
    
    // Process profile pictures
    const processedResults = await Promise.all(results.map(async (employee) => {
      if (employee.profilePicture) {
        try {
          // Assuming the profile picture path is relative to your project root
          const filePath = path.join(process.cwd(), employee.profilePicture.replace(/^\.\./, ''));
          const imageBuffer = await fs.readFile(filePath);
          const base64Image = imageBuffer.toString('base64');
          return {
            ...employee,
            profilePicture: base64Image // Return just the base64 string without the prefix
          };
        } catch (error) {
          console.error(`Error processing profile picture for employee ${employee._id}:`, error);
          return {
            ...employee,
            profilePicture: null
          };
        }
      }
      return employee;
    }));

    // Count total documents
    const totalPipeline = aggregationPipeline.slice(0, -3);  // Remove sort, skip and limit
    totalPipeline.push({ $count: 'total' });
    const totalResult = await Employee.aggregate(totalPipeline);
    const total = totalResult.length > 0 ? totalResult[0].total : 0;

    return {
      total,
      page: fetchAll ? 1 : pageNum,
      pageSize: fetchAll ? total : limitNum,
      results: processedResults
    };
  } catch (error) {
    console.error('Error in findAllWithSearchAndPagination:', error);
    throw new Error('Database query failed');
  }
};

const findById = async (id) => {
  return await Employee.findOne({ _id: id })
    .populate({ path: 'skillset', select: 'name' })
    .populate({ path: 'role', select: 'roleName' })
    .populate({
      path: 'Projects.projectId',
      populate: {
        path: 'project',
        select: 'name',
      },
    });
};

// Other functions...


const create = async (teamMemberData) => {
  const newTeamMember = new Employee(teamMemberData);
  return await newTeamMember.save();
};

const updateById = async (id, teamMemberData) => {
  return await Employee.findOneAndUpdate({ _id: id}, teamMemberData, { new: true });
};

const deleteById = async (id, terminationDate) => {
  return await Employee.findByIdAndUpdate(
    id, 
    { 
      visibility: false,
      status: 'Inactive',
      terminationDate: new Date(terminationDate)
    }, 
    { new: true }
  );
};


const getTotalCurrentEmployees = async () => {
  return await Employee.aggregate([
    { $match: { visibility: true } },
    { $count: 'totalCurrentEmployees' }
  ]);
};

const getTotalResignedEmployees = async () => {
  return await Employee.aggregate([
    { $match: { visibility: false } },
    { $count: 'totalResignedEmployees' }
  ]);
};

const getTotalAllEmployees = async () => {
  return await Employee.aggregate([
    { $count: 'totalAllEmployees' }
  ]);
};


const getTotalBillableEmployees = async () => {
  return await Employee.aggregate([
    { $match: { visibility: true, billableStatus: 'Billable' } },
    { $count: 'totalBillableEmployees' }
  ]);
};

const getTotalNonBillableEmployees = async () => {
  return await Employee.aggregate([
    { $match: { visibility: true, billableStatus: 'NonBillable' } },
    { $count: 'totalNonBillableEmployees' }
  ]);
};
const getTotalRemoteEmployees = async () => {
  return await Employee.aggregate([
    { $match: { visibility: true, workType: 'Remote' } },
    { $count: 'totalRemoteEmployees' }
  ]);
};

const getTotalResourceOnBench = async () => {
  return await Employee.aggregate([
    // Match employees with visibility true and role name 'Employee'
    {
      $match: {
        visibility: true
      }
    },
    // Lookup to get the role name
    {
      $lookup: {
        from: 'roles',
        localField: 'role',
        foreignField: '_id',
        as: 'roleInfo'
      }
    },
    // Unwind the roleInfo array
    {
      $unwind: '$roleInfo'
    },
    // Match only employees with role name 'Employee'
    {
      $match: {
        'roleInfo.roleName': 'Employee'
      }
    },
    // Add a field to check if the employee is on bench
    {
      $addFields: {
        isOnBench: {
          $cond: {
            if: { $eq: [{ $size: '$Projects' }, 0] },
            then: true,
            else: {
              $allElementsTrue: {
                $map: {
                  input: '$Projects',
                  as: 'project',
                  in: { $eq: ['$$project.benchStatus', 'On bench'] }
                }
              }
            }
          }
        }
      }
    },
    // Match only employees who are on bench
    {
      $match: {
        isOnBench: true
      }
    },
    // Count the total
    {
      $count: 'totalResourcesOnBench'
    }
  ]);
};

const updateEmployeeMessageStatus = async (employeeId, date, status) => {
  try {
    // Parse the input date
    const inputDate = new Date(date);
    
    // Set the time to the start of the day in UTC (00:00:00)
    const startOfDay = new Date(Date.UTC(inputDate.getUTCFullYear(), inputDate.getUTCMonth(), inputDate.getUTCDate()));
    
    // Set the time to the end of the day in UTC (23:59:59.999)
    const endOfDay = new Date(Date.UTC(inputDate.getUTCFullYear(), inputDate.getUTCMonth(), inputDate.getUTCDate(), 23, 59, 59, 999));

    const result = await Employee.findOneAndUpdate(
      { 
        _id: employeeId, 
        'messages.date': {
          $gte: startOfDay,
          $lt: endOfDay
        }
      },
      {
        $set: {
          'messages.$[elem].status': status
        }
      },
      { 
        new: true,
        arrayFilters: [{ 'elem.date': { $gte: startOfDay, $lt: endOfDay } }]
      }
    );

    return result;
  } catch (error) {
    console.error('Error updating employee message status in repository:', error);
    throw error;
  }
};




module.exports = {
  findAllWithSearchAndPagination,
  findById,
  create,
  updateById,
  deleteById,
  getTotalCurrentEmployees,
  getTotalResignedEmployees,
  getTotalAllEmployees,
  getTotalBillableEmployees,
  getTotalNonBillableEmployees,
  getTotalRemoteEmployees,
  getTotalResourceOnBench,
  updateEmployeeMessageStatus
};