const timesheetRepository = require('../timesheet.repository');
const Timesheet = require('../../../database/models/timesheetSchema');
const TimeEntry = require('../../../database/models/timeEntrySchema');
const Project = require('../../../database/models/project/projectSchema');
// const ProjectTeamMember = require('../../../database/models/project/projectTeamMemberSchema');

const addTimeEntry = async (timeEntryData) => {
  const { employeeId, date, startTime, endTime, ...entryData } = timeEntryData;


 

  // Check for overlapping time entries
  const overlappingEntries = await timesheetRepository.findOverlappingEntries(employeeId, date, startTime, endTime);
  if (overlappingEntries.length > 0) {
    throw new Error('TIME_OVERLAPS');
  }

  const timeEntry = await timesheetRepository.addTimeEntry({ employeeId, date, startTime, endTime, ...entryData });
  const timesheet = await timesheetRepository.findOrCreateTimesheet(employeeId, date);
  await timesheetRepository.updateTimesheet(timesheet._id, timeEntry._id);
  return timeEntry;
};

const getMonthlyHours = async (employeeId, projectId, taskId, year, month, day) => {
  let startDate, endDate;

  if (year && month && day) {
    // Specific day in a month
    startDate = new Date(year, month - 1, day);
    endDate = new Date(year, month - 1, day + 1);
  } else if (year && month) {
    // Specific month in a year
    startDate = new Date(year, month - 1, 1);
    endDate = new Date(year, month, 0);
  } else if (year) {
    // Specific year
    startDate = new Date(year, 0, 1);
    endDate = new Date(year + 1, 0, 1);
  } else {
    throw new Error('Invalid date parameters');
  }

  const timesheets = await Timesheet.find({
    employeeId,
    date: { $gte: startDate, $lte: endDate },
  }).populate('entryIds');

  let totalApprovedHours = 0; // Total hours for approved entries
  let totalLoggedHours = 0; // Total hours for all entries
  const dailyLoggedHours = {}; // Logged hours per day
  const dailyApprovedHours = {}; // Approved hours per day

  // Initialize the daily status object
  const dailyStatus = {};

  timesheets.forEach(timesheet => {
    const date = timesheet.date.toISOString().split('T')[0];

    // Update the daily status object
    dailyStatus[date] = {
      approved: timesheet.approved,
      submitted: timesheet.submitted
    };

    timesheet.entryIds.forEach(entry => {
      const entryDate = entry.date.toISOString().split('T')[0];
      const duration = entry.duration;

      // Calculate logged hours for all entries
      const hoursLogged = parseFloat((duration / 3600).toFixed(2));

      // Add to daily logged hours
      if(entry.submit == 'true' && (!projectId || entry.projectId.toString() === projectId) && (!taskId || entry.taskId.toString() === taskId)){
        dailyLoggedHours[entryDate] = (dailyLoggedHours[entryDate] || 0) + hoursLogged;
        totalLoggedHours += hoursLogged; // Add to total logged hours
      }
      
      // If entry is approved, add to approved hours
      if (entry.submit == 'true' && entry.approved && (!projectId || entry.projectId.toString() === projectId) && (!taskId || entry.taskId.toString() === taskId)) {
        dailyApprovedHours[entryDate] = (dailyApprovedHours[entryDate] || 0) + hoursLogged;
        totalApprovedHours += hoursLogged;
      }
    });
  });

  // Combine logged hours, approved hours, and statuses into an array
  const result = Object.keys(dailyLoggedHours).map(date => ({
    date,
    loggedHours: dailyLoggedHours[date] || 0, // Total logged hours for the day, rounded to 2 decimal places
    approvedHours: dailyApprovedHours[date] || 0, // Approved hours for the day, rounded to 2 decimal places
    approved: dailyStatus[date]?.approved || false,
    submitted: dailyStatus[date]?.submitted || false
  }));
  

  return {
    dailyHours: result,
    totalApprovedHours,
    totalLoggedHours // Return total logged hours (approved + unapproved)
  };
};





// Service method to get the daily timesheet
const getDailyTimesheet = async (employeeId, date,startTime, endTime,timeentryId) => {
  let overlappingEntries = await timesheetRepository.findOverlappingEntries(employeeId, date, startTime, endTime);
  console.log('overlap Entries : ',overlappingEntries);
  overlappingEntries = overlappingEntries.filter(entry => entry._id.toString() !== timeentryId);
  if (overlappingEntries.length > 0) {
    throw new Error('TIME_OVERLAPS');
  }
  return Timesheet.findOne({ employeeId, date }).exec();
};


const getTimeEntries = async (timesheetId) => {
  const timesheet = await Timesheet.findById(timesheetId)
    .populate({
      path: 'entryIds',
      populate: {
        path: 'projectId',
        select: 'name tam',
        populate: {
          path: 'tam',
          select: 'name email'  // Adjust these fields based on what you want to show for the TAM
        }
      }
    })
    .exec();

  if (!timesheet) {
    throw new Error('Timesheet not found');
  }

  return timesheet.entryIds.map(entry => ({
    ...entry.toObject(),
    project: entry.projectId ? {
      name: entry.projectId.name,
      tam: entry.projectId.tam ? {
        name: entry.projectId.tam.name,
        email: entry.projectId.tam.email
      } : null
    } : null
  }));
};

// Service method to delete a specific time entry
const deleteTimeEntry = async (timeEntryId, timesheetId) => {
  // Remove the time entry from the timesheet's entryIds array
  await timesheetRepository.removeTimeEntryFromTimesheet(timesheetId, timeEntryId);

  // Delete the time entry document
  await timesheetRepository.deleteTimeEntry(timeEntryId);
};

// Service method to update a specific time entry
const updateTimeEntry = async (timeEntryId, updates) => {
  // Update the time entry document
  return timesheetRepository.updateTimeEntry(timeEntryId, updates);
};

const submitTimesheet = async (timesheetId, employeeId, userRole) => {
  const timesheet = await Timesheet.findById(timesheetId);
  
  // Check if timesheet exists
  if (!timesheet) throw new Error('Timesheet not found');
  
  // Check if the employee is authorized to submit this timesheet
  if (timesheet.employeeId.toString() !== employeeId) throw new Error('Unauthorized');
  
  // Check if timesheet has already been submitted
  if (timesheet.submitted) throw new Error('Timesheet already submitted');

  // Mark the timesheet as submitted
  timesheet.submitted = true;
  
  // Add the submission details to the history array
  timesheet.history.push({
    employeeId: employeeId,
    status: 'Submitted',
    userRole: userRole,
    updatedAt: new Date()
  });

  // Fetch all associated time entries by their IDs
  const timeEntries = await TimeEntry.find({ _id: { $in: timesheet.entryIds } });

  // Update all time entries' submit field to true
  for (let entry of timeEntries) {
    entry.submit = 'true';
    await entry.save();
  }

  // Save the updated timesheet
  await timesheet.save();
  
  return timesheet;
};


const approveTimesheet = async (timesheetId) => {
  const timesheet = await Timesheet.findById(timesheetId);
  if (!timesheet) throw new Error('Timesheet not found');
  if (!timesheet.submitted) throw new Error('Timesheet not submitted');
  if (timesheet.approved) throw new Error('Timesheet already approved');
  timesheet.approved = true;


  await timesheet.save();

  // Approve all associated time entries
  await TimeEntry.updateMany(
    { _id: { $in: timesheet.entryIds } },
    { $set: { approved: true } }
  );

  return timesheet;
};

const getTamTimesheets = async (tamId, startDate, endDate) => {
  // Find all projects where the given TAM is assigned
  const projects = await Project.find({ tam: tamId });
  const projectIds = projects.map(project => project._id);

  // Get timesheets for employees working on these projects within the date range
  const timesheets = await Timesheet.find({
    date: { $gte: startDate, $lte: endDate }
  }).populate('employeeId', 'name email employeeId')
    .populate({
      path: 'entryIds',
      match: { projectId: { $in: projectIds } },
      populate: {
        path: 'projectId',
        select: 'name tam'
      }
    });

  // Process timesheets to calculate total hours per employee and filter by project TAM
  const processedTimesheets = timesheets.reduce((acc, timesheet) => {
    const validEntries = timesheet.entryIds.filter(entry => entry.projectId.tam.toString() === tamId);
    
    if (validEntries.length > 0) {
      const totalHours = validEntries.reduce((sum, entry) => sum + entry.duration, 0);
      
      acc.push({
        Id: timesheet.employeeId._id,
        employeeName: timesheet.employeeId.name,
        employeeEmail: timesheet.employeeId.email,
        date: timesheet.date,
        totalHours: totalHours / 3600, // Convert seconds to hours
        employeeId: timesheet.employeeId.employeeId,
        projects: validEntries.map(entry => ({
          projectId: entry.projectId._id,
          projectName: entry.projectId.name,
          duration: entry.duration / 3600, // Convert seconds to hours
          description: entry.description
        }))
      });
    }
    
    return acc;
  }, []);

  return processedTimesheets;
};


const approveTimeEntries = async (timesheetId, timelogIds, tamId, userRole) => {


  const timesheet = await timesheetRepository.getTimesheetById(timesheetId);

  if (!timesheet.submitted) {
    throw new Error('Cannot approve time entries for an unsubmitted timesheet');
  }
  // Approve the specified time entries
  await timesheetRepository.approveTimeEntries(timelogIds, tamId, userRole);

  // Check if all time entries are approved
  const allEntriesApproved = await timesheetRepository.areAllEntriesApproved(timesheetId);

  // If all entries are approved, update the timesheet status
  if (allEntriesApproved) {
    await timesheetRepository.approveTimesheet(timesheetId);
    timesheet.approved = true;
  }

  return timesheet;
};


const rejectTimeEntries = async (timesheetId, timelogIds, remarks, tamId, userRole) => {
  const timesheet = await timesheetRepository.getTimesheetById(timesheetId);

  if (!timesheet.submitted) {
    throw new Error('Cannot approve time entries for an unsubmitted timesheet');
  }

  // Reject the specified time entries
  await timesheetRepository.rejectTimeEntries(timelogIds, remarks, tamId, userRole);

  // Check if all time entries are approved
  const allEntriesApproved = await timesheetRepository.areAllEntriesApproved(timesheetId);

  // If all entries are approved, update the timesheet status
  if (allEntriesApproved) {
    await timesheetRepository.approveTimesheet(timesheetId);
    timesheet.approved = true;
  }

  return timesheet;
};

const getTimeEntryHistories = async (employeeId) => {
  const timeEntries = await timesheetRepository.getTimeEntryHistories(employeeId);

  const notifications = timeEntries.flatMap(entry => {
    return entry.history.map(historyItem => {
      // const date = historyItem.updatedAt.toLocaleString('en-US', {
      //   year: 'numeric',
      //   month: 'long',
      //   day: 'numeric',
      //   hour: '2-digit',
      //   minute: '2-digit'
      // });

      return {
        id: historyItem._id, // Assuming each history item has an _id
        status: historyItem.status,
        message: historyItem.status === 'approved'
          ? `Your ${entry.taskId} from ${entry.projectId.name} has been approved on`
          : `Your ${entry.taskId} from ${entry.projectId.name} has been rejected on`,
        read: historyItem.read || false,
        updatedAt: historyItem.updatedAt
      };
    });
  }).sort((a, b) => new Date(b.updatedAt) - new Date(a.updatedAt));

  return notifications;
};

// Add a new function to mark notifications as read
const markNotificationsAsRead = async (employeeId, notificationIds) => {
  await TimeEntry.updateMany(
    { employeeId, 'history._id': { $in: notificationIds } },
    { $set: { 'history.$[elem].read': true } },
    { arrayFilters: [{ 'elem._id': { $in: notificationIds } }] }
  );

  return { success: true };
};

// Add a new function to get unread notification count
const getUnreadNotificationCount = async (employeeId) => {
  const timeEntries = await TimeEntry.find({ employeeId });
  const unreadCount = timeEntries.reduce((count, entry) => {
    return count + entry.history.filter(item => !item.read).length;
  }, 0);

  return unreadCount;
};


const startTimer = async (employeeId, projectId, Category, task) => {
  // Find the employee and start the timer
  const updatedEmployee = await timesheetRepository.startTimer(employeeId, projectId, Category, task);
  return updatedEmployee;
};

const stopTimer = async (employeeId) => {
  // Find the employee and stop the timer
  const updatedEmployee = await timesheetRepository.stopTimer(employeeId);
  return updatedEmployee;
};

const getEmployeeWorkedHours = async (projectId, startDate, endDate) => {

  const timeEntries = await timesheetRepository.getVisibleSubmittedApprovedEntries(projectId, startDate, endDate);

  // Initialize structures to hold the results
  const workedHours = {};
  const categoryHours = {
    'PM': 0,
    'QA': 0,
    'BA': 0,
    'UI/UX': 0,
    'Research': 0,
    'Development': 0,
    'Meeting': 0,
    'Others': 0
  };

  // Process entries to aggregate worked hours and categories
  timeEntries.forEach(entry => {
    const employeeId = entry.employeeId._id.toString(); // Use populated employeeId
    const employeeName = entry.employeeId.name; // Get name from populated employee
    const duration = entry.duration; // Assuming duration field holds the worked hours

    // Add or update worked hours for the employee
    if (!workedHours[employeeId]) {
      workedHours[employeeId] = {
        name: employeeName,
        totalHours: 0
      };
    }
    workedHours[employeeId].totalHours += duration;

    // Add hours to categories
    categoryHours[entry.Category] += duration; // Assuming Category is the category field in your entry
  });

  return {
    workedHours,
    categoryHours
  };
};




const approveProjectEntries = async (projectIds, date, employeeId,timesheetId) => {
  const updatedEntries = await timesheetRepository.approveProjectEntries(projectIds, date, employeeId);

  // Check if all time entries are approved
  const allEntriesApproved = await timesheetRepository.areAllEntriesApproved(timesheetId);

  // If all entries are approved, update the timesheet status
  if (allEntriesApproved) {
    const timesheet = await Timesheet.findById(timesheetId);
    timesheet.approved = true;
    await timesheetRepository.approveTimesheet(timesheetId);
    
  }

  return {
    message: 'Project time entries approved successfully',
    modifiedCount: updatedEntries.modifiedCount,
  };
};



const getPendingApprovalsForTAM = async (tamId,startDate, endDate) => {
  try {
    // Fetch the pending timesheets
    
    const pendingTimesheets = await timesheetRepository.findPendingTimesheetsForTAM(tamId,startDate, endDate);
    // Fetch all projects managed by the current TAM
    const projectsManagedByTAM = await Project.find({ tam: tamId }).select('_id');
    const projectIdsManagedByTAM = projectsManagedByTAM.map(project => project._id.toString());

    // Map timesheets with total pending hours and project-specific details
    const timesheetData = pendingTimesheets.map(timesheet => {
      // Filter entries that are not yet approved and belong to projects managed by this TAM
      const unapprovedEntriesForTAM = timesheet.entryIds.filter(entry => {
        // Entry must be unapproved and belong to a TAM-managed project
        return !entry.approved && projectIdsManagedByTAM.includes(entry.projectId._id.toString());
      });

      // Calculate total hours for the unapproved entries specific to this TAM
      const totalPendingHours = unapprovedEntriesForTAM.reduce((total, entry) => total + entry.duration, 0);

      // Create a map of project IDs for aggregating project-specific hours for pending approval
      const projects = unapprovedEntriesForTAM.reduce((acc, entry) => {
        const projectId = entry.projectId._id.toString();

        // If project isn't in the accumulator, add it with initial hours
        if (!acc[projectId]) {
          acc[projectId] = {
            projectId: entry.projectId._id,
            projectName: entry.projectId.name,
            totalHoursForProject: 0,
          };
        }

        // Add the entry's duration to the total hours for that project
        acc[projectId].totalHoursForProject += entry.duration;

        return acc;
      }, {});

      // Only return data if there are unapproved entries for this TAM's projects
      if (totalPendingHours > 0) {
        return {
          employeeId: timesheet.employeeId._id,
          employeeName: timesheet.employeeId.name,
          timesheetId: timesheet._id,
          date: timesheet.date,
          totalPendingHours,  // Hours pending approval from this TAM
          projects: Object.values(projects),  // Project-specific pending hours
        };
      }

      return null; // Filter out timesheets without pending hours for this TAM
    }).filter(timesheet => timesheet !== null);  // Remove null values

    return timesheetData;
  } catch (error) {
    throw new Error(`Failed to fetch pending approvals: ${error.message}`);
  }
};





module.exports = {
  addTimeEntry,
  getMonthlyHours,
  getDailyTimesheet,
  getTimeEntries,
  deleteTimeEntry,
  updateTimeEntry,
  submitTimesheet,
  approveTimesheet,
  getTamTimesheets,
  approveTimeEntries,
  rejectTimeEntries,
  getTimeEntryHistories,
  markNotificationsAsRead,
  getUnreadNotificationCount,
  startTimer,
  stopTimer,
  getEmployeeWorkedHours,
  approveProjectEntries,
  getPendingApprovalsForTAM
};
