const timesheetService = require('./Services/timesheet.service');



// Controller method to add a time entry
const addTimeEntry = async (req, res) => {
  try {
    const timeEntryData = req.body;
    const result = await timesheetService.addTimeEntry(timeEntryData);
    res.status(201).json(result);
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

// Controller method to get total hours worked in a specific month
const getMonthlyHours = async (req, res) => {
  try {
    const { employeeId, projectId, taskId, year, month, day } = req.query;
    const result = await timesheetService.getMonthlyHours(employeeId, projectId, taskId, year, month, day);
    res.status(200).json(result);
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

// Controller method to get all time entries for a specific day
const getDailyEntries = async (req, res) => {
  try {
    const { employeeid, timesheetdate } = req.params;
    const date = new Date(timesheetdate);

    // Fetch the daily timesheet
    const timesheet = await timesheetService.getDailyTimesheet(employeeid, date);

    if (!timesheet) {
      return res.status(404).json({ message: 'Timesheet not found' });
    }

    // Populate entries
    const entries = await timesheetService.getTimeEntries(timesheet._id);

    res.status(200).json({
      timesheetid: timesheet._id,
      submitted: timesheet.submitted,
      approved: timesheet.approved,
      date: timesheet.date,
      entries: entries,
      totalduration: entries.reduce((acc, entry) => acc + entry.duration, 0)
    });
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

// Controller method to delete a specific time entry log
const deleteTimeEntry = async (req, res) => {
  try {
    const { employeeid, timesheetdate, timeentryId } = req.params;
    const date = new Date(timesheetdate);

    // Fetch the daily timesheet
    const timesheet = await timesheetService.getDailyTimesheet(employeeid, date);

    if (!timesheet) {
      return res.status(404).json({ message: 'Timesheet not found' });
    }

    // Remove time entry from the timesheet and delete the time entry
    await timesheetService.deleteTimeEntry(timeentryId, timesheet._id);
    return res.status(201).json({ message: `${timeentryId} is successfully deleted` });
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

// Controller method to update a specific time entry log
const updateTimeEntry = async (req, res) => {
  try {
    const { employeeid, timesheetdate, timeentryId } = req.params;
    const updates = req.body;
    const date = new Date(timesheetdate);

    if (updates.employeeId) {
      delete updates.employeeId;
    }

    if (updates.approved) {
      delete updates.approved;
    }

    if (updates.rejected) {
      delete updates.rejected;
    }
    // Fetch the daily timesheet
    const timesheet = await timesheetService.getDailyTimesheet(employeeid, date,updates.startTime, updates.endTime,timeentryId);

    if (!timesheet) {
      return res.status(404).json({ message: 'Timesheet not found' });
    }

    // Update the time entry
    const updatedEntry = await timesheetService.updateTimeEntry(timeentryId, updates);
    res.status(200).json(updatedEntry);
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

const submitTimesheet = async (req, res) => {
  try {
    const { timesheetId } = req.params;
    const { employeeId, userRole } = req.body;
    const result = await timesheetService.submitTimesheet(timesheetId, employeeId, userRole);
    res.status(200).json(result);
  } catch (error) {
    res.status(400).json({ message: error.message });
  }
};

const approveTimeEntries = async (req, res) => {
  try {
    const { timesheetId } = req.params;
    const { timelogIds, tamId, userRole } = req.body;
    const result = await timesheetService.approveTimeEntries(timesheetId, timelogIds, tamId, userRole);

    res.status(200).json(result);
  } catch (error) {
    res.status(500).json({ error: error.message });
  }
};

const rejectTimeEntries = async (req, res) => {
  try {
    const { timesheetId } = req.params;
    const { timelogIds, remarks, tamId, userRole } = req.body;

    const result = await timesheetService.rejectTimeEntries(timesheetId, timelogIds, remarks, tamId, userRole);

    res.status(200).json(result);
  } catch (error) {
    res.status(500).json({ error: error.message });
  }
};

const getTimesheetEntries = async (req, res) => {
  try {
    const { timesheetId } = req.params;
    const entries = await timesheetService.getTimeEntries(timesheetId);
    res.status(200).json(entries);
  } catch (error) {
    res.status(400).json({ message: error.message });
  }
};

const getTamTimesheets = async (req, res) => {
  try {
    const { tamId } = req.params;
    const { startDate, endDate } = req.query;

    if (!startDate || !endDate) {
      return res.status(400).json({ message: 'Start date and end date are required' });
    }

    const timesheets = await timesheetService.getTamTimesheets(
      tamId,
      new Date(startDate),
      new Date(endDate)
    );

    res.status(200).json(timesheets);
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

const getTimeEntryHistories = async (req, res) => {
  try {
    const { employeeId } = req.params;

    const timeEntries = await timesheetService.getTimeEntryHistories(employeeId);
    res.status(200).json(timeEntries);
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

const markNotificationsAsRead = async (req, res) => {
  try {
    const { employeeId } = req.params;
    const { notificationIds } = req.body;
    const result = await timesheetService.markNotificationsAsRead(employeeId, notificationIds);
    res.status(200).json(result);
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

const getUnreadNotificationCount = async (req, res) => {
  try {
    const { employeeId } = req.params;
    const count = await timesheetService.getUnreadNotificationCount(employeeId);
    res.status(200).json({ count });
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};


const startTimer = async (req, res) => {
  try {
    const { employeeId, projectId, Category, task } = req.body;
    const updatedEmployee = await timesheetService.startTimer(employeeId, projectId, Category, task);
    
    res.status(200).json({
      message: 'Timer started successfully',
      data: updatedEmployee,
    });
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};


const stopTimer = async (req, res) => {
  try {
    const { employeeId } = req.body;
    const updatedEmployee = await timesheetService.stopTimer(employeeId);
    
    res.status(200).json({
      message: 'Timer stopped successfully',
      data: updatedEmployee,
    });
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

const getWorkedHours = async (req, res) => {
  const { projectId, startDate, endDate } = req.query;

  try {
    // Convert startDate and endDate to Date objects if provided
    const start = startDate ? new Date(startDate) : null; // Set to null if not provided
    const end = endDate ? new Date(endDate) : null; // Set to null if not provided
    
    const results = await timesheetService.getEmployeeWorkedHours(projectId, start, end);
    res.status(200).json(results);
  } catch (error) {
    console.error('Error fetching worked hours:', error);
    res.status(500).json({ message: 'Internal server error' });
  }
};


const approveProjectEntries = async (req, res) => {
  try {
    const { projectIds, date, employeeId,timesheetId } = req.body;
    const result = await timesheetService.approveProjectEntries(projectIds, date, employeeId,timesheetId);
    res.status(200).json(result);
  } catch (error) {
    res.status(500).json({ message: error.message });
  }
};

const getPendingApprovals = async (req, res) => {
  const { tamId} = req.params;
  const { startDate, endDate } = req.query;
  

  
  try {
    const pendingApprovals = await timesheetService.getPendingApprovalsForTAM(tamId,startDate, endDate);
    res.status(200).json({ success: true, data: pendingApprovals });
  } catch (error) {
    res.status(500).json({ success: false, message: error.message });
  }
};


module.exports = {
  addTimeEntry,
  getMonthlyHours,
  getDailyEntries,
  deleteTimeEntry,
  updateTimeEntry,
  submitTimesheet,
  approveTimeEntries,
  getTimesheetEntries,
  getTamTimesheets,
  rejectTimeEntries,
  getTimeEntryHistories,
  markNotificationsAsRead,
  getUnreadNotificationCount,
  startTimer,
  stopTimer,
  getWorkedHours,
  approveProjectEntries,
  getPendingApprovals

};
