const TimeEntry = require('../../database/models/timeEntrySchema');
const Timesheet = require('../../database/models/timesheetSchema');
const Employee = require('../../database/models/employeeSchema');
const Project  = require('../../database/models/project/projectSchema');

const addTimeEntry = async (timeEntryData) => {
  const timeEntry = new TimeEntry(timeEntryData);
  await timeEntry.save();
  return timeEntry;
};

const findOrCreateTimesheet = async (employeeId, date) => {
  let timesheet = await Timesheet.findOne({ employeeId, date });
  if (!timesheet) {
    timesheet = new Timesheet({ employeeId, date, entryIds: [] });
    await timesheet.save();
  }
  return timesheet;
};

const updateTimesheet = async (timesheetId, entryId) => {
  const timesheet = await Timesheet.findById(timesheetId);
  if (timesheet) {
    timesheet.entryIds.push(entryId);
    await timesheet.save();
    // Fetch all time entries associated with this timesheet
    const timeEntries = await TimeEntry.find({ _id: { $in: timesheet.entryIds } });

    // Check if all time entries have submit: true
    const allSubmitted = timeEntries.every(entry => entry.submit === 'true');
    const allapprove = timeEntries.every(entry => (entry.approved == true || entry.rejected == true));

    // Set the submitted status of the timesheet
    timesheet.submitted = allSubmitted;
    timesheet.approved = allapprove;
    

    await timesheet.save();
    return timesheet;
  }
  throw new Error('Timesheet not found');
};

const getFilteredTimeEntries = async (employeeId, projectId, taskId, startDate, endDate) => {
  const filter = {
    date: {
      $gte: startDate,
      $lt: endDate, // Using $lt to ensure end of the day is considered
    }
  };

  if (employeeId) {
    filter.employeeId = employeeId;
  }
  if (projectId) {
    filter.projectId = projectId;
  }
  if (taskId) {
    filter.taskId = taskId;
  }

  return await TimeEntry.find(filter);
};

const findOverlappingEntries = async (employeeId, date, startTime, endTime) => {
  return await TimeEntry.find({
    employeeId,
    date,
    $or: [
      {
        $and: [
          { startTime: { $lt: endTime } },
          { endTime: { $gt: startTime } }
        ]
      }
    ]
  });
};

// Repository method to get total hours worked in a specific month for an employee
const getMonthlyHours = async (employeeId, startDate, endDate) => {
  const timesheets = await Timesheet.find({
    employeeId,
    date: {
      $gte: startDate,
      $lte: endDate,
    },
  }).populate('entryIds').exec();

  let totalSeconds = 0;
  timesheets.forEach(sheet => {
    sheet.entryIds.forEach(entry => {
      totalSeconds += entry.duration;
    });
  });

  return totalSeconds;
};

// Repository method to remove a time entry from a timesheet
const removeTimeEntryFromTimesheet = async (timesheetId, timeEntryId) => {
  const timesheet = await Timesheet.findById(timesheetId).exec();
  if (timesheet) {
    timesheet.entryIds.pull(timeEntryId);
    await timesheet.save();

  }
};

// Repository method to delete a time entry document
const deleteTimeEntry = async (timeEntryId) => {
  return TimeEntry.findByIdAndDelete(timeEntryId).exec();
};

// Repository method to update a time entry document
const updateTimeEntry = async (timeEntryId, updates) => {
  return TimeEntry.findByIdAndUpdate(timeEntryId, updates, { new: true }).exec();
};


const approveTimeEntries = async (timelogIds, tamId, userRole) => {
  const update = {

    $set: { approved: true, rejected: false },
    $push: {
      history: {
        employeeId: tamId,  // Employee or user ID who approved the entry
        status: 'approved', // Status of the action
        userRole: userRole, // Role of the approving user
        updatedAt: new Date(), // Timestamp of the approval
      },
    },
  };

  await TimeEntry.updateMany(
    { 
      _id: { $in: timelogIds },
      submit: 'true',         // Only update if submit is "true" as a string
      approved: false,        // Only if it's not already approved
      rejected: false         // Only if it's not already rejected
    },
    update
  );
  
};


const rejectTimeEntries = async (timelogIds, remarks, tamId, userRole) => {
  const update = {
    $set: { 
      rejected: true, 
      approved: false,
      remarks: remarks  
    },
    $push: {
      history: {
        employeeId: tamId,
        status: 'rejected',
        userRole: userRole,
        remarks: remarks,  // Add remarks to history
        updatedAt: new Date(),
      },
    },
  };

  await TimeEntry.updateMany(
    { 
      _id: { $in: timelogIds },
      submit: 'true',         // Only update if submit is "true" as a string
      approved: false,        // Only if it's not already approved
      rejected: false         // Only if it's not already rejected
    },
    update
  );
  
};


const getTimesheetById = async (timesheetId) => {
  return await Timesheet.findById(timesheetId).populate('entryIds');
};

const areAllEntriesApproved = async (timesheetId) => {
  const timesheet = await Timesheet.findById(timesheetId).populate('entryIds');
  return timesheet.entryIds.every(entry => entry.approved || entry.rejected);
};



const approveTimesheet = async (timesheetId) => {
  await Timesheet.findByIdAndUpdate(timesheetId, { approved: true });
};

const getTimeEntryHistories = async (employeeId) => {
  return await TimeEntry.find({ employeeId })
    .populate('projectId', 'name')
    .sort({ 'history.updatedAt': -1 })
    .exec();
};


const startTimer = async (employeeId, projectId, Category, task) => {
  const employee = await Employee.findById(employeeId);
  if (!employee) {
    throw new Error('Employee not found');
  }

  if (employee.timer.length) {
    throw new Error('Timer is already running');
  }
 
  // Start a new timer entry
  const newTimer = {
    isRunning: true,
    project: projectId,
    startTime: new Date(),
    Category: Category || '',
    task: task || '',
  };

  // Add the new timer to the employee's timer array
  employee.timer.push(newTimer);

  // Save the employee document
  await employee.save();

  return employee;
};

const stopTimer = async (employeeId) => {
  const employee = await Employee.findById(employeeId);
  if (!employee) {
    throw new Error('Employee not found');
  }

  // Check if there is any running timer (if the array is not empty)
  if (!employee.timer.length) {
    throw new Error('No running timer found');
  }

  // Clear the timer array to stop the timer and reset all values
  employee.timer = [];

  // Save the employee document
  await employee.save();

  return employee;
};

const getVisibleSubmittedApprovedEntries = async (projectId, startDate, endDate) => {
  try {
    const query = {
      submit: 'true',
      approved: true,
      projectId: projectId, // Ensure projectId is always included in the query
    };

    if (startDate && endDate) {
      query.date = { $gte: startDate, $lte: endDate };
    }
   
    return await TimeEntry.find(query).populate('employeeId', 'name'); // Populate only the 'name' field
  } catch (error) {
    console.error('Error fetching time entries:', error);
    throw new Error('Database error');
  }
};




const approveProjectEntries = async (projectIds, date, employeeId) => {
  const startOfDay = new Date(date);
  startOfDay.setHours(0, 0, 0, 0);
  const endOfDay = new Date(date);
  endOfDay.setHours(23, 59, 59, 999);

  const updatedEntries = await TimeEntry.updateMany(
    {
      employeeId: employeeId,
      date: { $gte: startOfDay, $lte: endOfDay },
      projectId: { $in: projectIds },
      submit: 'true', 
      approved: false ,
      rejected: false,
    },
    {
      $set: { approved: true },
      $push: {
        history: {
          employeeId: employeeId,
          status: 'approved',
          userRole: 'system', // You may want to pass the user role from the service
          updatedAt: new Date(),
        },
      },
    }
  );

  return updatedEntries;
};


const findPendingTimesheetsForTAM = async (tamId, startDate, endDate) => {
  try {
    // Find projects where TAM is the given employee
    const projectsManagedByTAM = await Project.find({ tam: tamId }).select('_id');
    const projectIds = projectsManagedByTAM.map(project => project._id.toString());
    

    // Build the query for pending timesheets
    const query = {
      submitted: true,  // Submitted by employee
      approved: false,  // Not fully approved
      employeeId: { $ne: tamId }  // Exclude timesheets where employeeId is the same as tamId
    };

    // Add date filter if startDate and endDate are provided
    if (startDate && endDate) {
      query.date = { $gte: new Date(startDate), $lte: new Date(endDate) };
      
    }
    
    // Fetch timesheets based on the query
    const pendingTimesheets = await Timesheet.find(query)
      .populate({
        path: 'entryIds',  // Populate TimeEntry references in entryIds
        populate: {
          path: 'projectId',  // Populate the projectId inside each TimeEntry
          model: 'Project'
        }
      })
      .populate('employeeId')  // Populate employee details
      .lean();  // Using lean for faster performance


    // Filter timesheets that have entries in the projects managed by the TAM
    const filteredTimesheets = pendingTimesheets.filter(timesheet => {
      return timesheet.entryIds.some(entry => {
        // Ensure entry has projectId populated and check if it matches the projectIds list
        return entry.projectId && projectIds.includes(entry.projectId._id.toString());
      });
    });


    // Return the filtered timesheets
    return filteredTimesheets;

  } catch (error) {
    throw new Error(`Failed to find pending timesheets: ${error.message}`);
  }
};



module.exports = {
  addTimeEntry,
  findOrCreateTimesheet,
  updateTimesheet,
  getFilteredTimeEntries,
  findOverlappingEntries,
  getMonthlyHours,
  removeTimeEntryFromTimesheet,
  deleteTimeEntry,
  updateTimeEntry,
  approveTimeEntries,
  areAllEntriesApproved,
  approveTimesheet,
  getTimesheetById,
  rejectTimeEntries,
  getTimeEntryHistories,
  startTimer,
  stopTimer,
  getVisibleSubmittedApprovedEntries,
  approveProjectEntries,
  findPendingTimesheetsForTAM

};
