const mongoose = require('mongoose');
const Extension = require('../../../database/models/project/extensionSchema');
const ProjectTeamMember = require('../../../database/models/project/projectTeamMemberSchema');
const Employee = require('../../../database/models/employeeSchema');
//const applySearchAndPagination = require('../../../utils/paginationAndSearch');

const createExtensionRequest = async (extensionData) => {
  const extension = new Extension(extensionData);
  return await extension.save();
};

const findAllPendingExtensions = async (search, page = 1, limit = 25, fetchAll = false) => {
  try {
    const trimmedSearch = search ? search.trim() : '';

    const aggregationPipeline = [
      { $match: { status: 'pending' } },
      {
        $lookup: {
          from: 'projectteammembers',
          localField: 'projectTeamMember',
          foreignField: '_id',
          as: 'projectTeamMember'
        }
      },
      { $unwind: '$projectTeamMember' },
      {
        $lookup: {
          from: 'projects',
          localField: 'projectTeamMember.project',
          foreignField: '_id',
          as: 'projectTeamMember.project'
        }
      },
      { $unwind: '$projectTeamMember.project' },
      {
        $lookup: {
          from: 'employees',
          localField: 'projectTeamMember.employee',
          foreignField: '_id',
          as: 'projectTeamMember.employee'
        }
      },
      { $unwind: '$projectTeamMember.employee' }
    ];
    
    if (trimmedSearch) {
      aggregationPipeline.push({
        $match: {
          $or: [
            { 'projectTeamMember.employee.employeeId': { $regex: trimmedSearch, $options: 'i' } },
            { 'projectTeamMember.employee.name': { $regex: trimmedSearch, $options: 'i' } },
            { 'projectTeamMember.employee.designation': { $regex: trimmedSearch, $options: 'i' } }
          ]
        }
      });
    }
    const countPipeline = [...aggregationPipeline, { $count: 'total' }];
    const [countResult] = await Extension.aggregate(countPipeline);
    const total = countResult ? countResult.total : 0;

    // Apply pagination
    if (!fetchAll) {
      const skip = (parseInt(page) - 1) * parseInt(limit);
      aggregationPipeline.push({ $skip: skip }, { $limit: parseInt(limit) });
    }

    // Execute the main query
    const extensions = await Extension.aggregate(aggregationPipeline);

    // Prepare the response
    const totalPages = fetchAll ? 1 : Math.ceil(total / parseInt(limit));
    const currentPage = fetchAll ? 1 : parseInt(page);
    const pageSize = fetchAll ? total : parseInt(limit);

    return {
      extensions,
      totalPages,
      currentPage,
      totalExtensions: total,
      pageSize
    };
  } catch (error) {
    console.error('Error in findAllPendingExtensions:', error);
    throw error;
  }
};


const updateExtensionStatus = async (id, status, userRole) => {
  const extension = await Extension.findById(id);
  if (!extension) {
    throw new Error('Extension request not found');
  }

  extension.status = status;
  extension.history.push({ status, userRole, updatedAt: new Date() });
  return await extension.save();
};

// New Employee Service
const employeeService = {
  updateProjectStatus: async (projectTeamMemberId) => {
    const projectTeamMember = await ProjectTeamMember.findById(projectTeamMemberId);

    if (!projectTeamMember) {
      throw new Error('Project team member not found');
    }

    const employee = await Employee.findById(projectTeamMember.employee);
    if (!employee) {
      throw new Error('Employee not found');
    }

    if (!employee.Projects || !Array.isArray(employee.Projects)) {
      throw new Error('Employee has no projects or invalid projects data');
    }

    const projectIndex = employee.Projects.findIndex(
      (p) => p && p.projectId && projectTeamMember._id && 
             p.projectId.toString() === projectTeamMember._id.toString()
    );

    if (projectIndex === -1) {
      throw new Error('Project not found in employee records');
    }

    employee.Projects[projectIndex].benchStatus = 'active';
    employee.Projects[projectIndex].active = true;

    await employee.save();
  }
};


const getExtensionHistoryByProjectId = async (projectId) => {
  // Check if projectId is valid
  if (!projectId || !mongoose.Types.ObjectId.isValid(projectId)) {
    throw new Error('Invalid project ID');
  }

  const extensions = await Extension.find()
    .populate({
      path: 'projectTeamMember',
      match: { project: projectId },
      populate: [
        {
          path: 'employee',
          select: 'name'
        },
        {
          path: 'project',
          select: 'name'
        }
      ]
    });

  // Filter out extensions where projectTeamMember is null (i.e., not matching the project)
  const filteredExtensions = extensions.filter(ext => ext.projectTeamMember !== null);

  const formattedHistory = filteredExtensions.flatMap(extension => {
    return extension.history.map(historyItem => {
      const roleName = historyItem.userRole || 'Unknown Role';
      const employeeName = extension.projectTeamMember?.employee?.name || 'Unknown Employee';
      // const date = historyItem.updatedAt.toLocaleString('en-US', {
      //   year: 'numeric',
      //   month: 'long',
      //   day: 'numeric',
      //   hour: '2-digit',
      //   minute: '2-digit'
      // });

      let action;
      switch (historyItem.status) {
      case 'pending':
        action = 'has requested for extension of the allocation of';
        break;
      case 'accepted':
        action = 'has extended the allocation of';
        break;
      case 'rejected':
        action = 'has rejected the allocation of';
        break;
      default:
        action = 'has updated the status for';
      }

      return {
        description: `${roleName} ${action} ${employeeName}`,
        date: historyItem.updatedAt // Store the date as a Date object for sorting
      };
    });
  });

  // Sort the formattedHistory array by date
  formattedHistory.sort((a, b) => a.date - b.date);

  return formattedHistory;
};

const getProjectTeamMemberById = async (id) => {
  return await ProjectTeamMember.findById(id).populate('employee');
};

const getPendingExtensionsByProjectId = async (projectId) => {
  // Check if projectId is valid
  if (!projectId) {
    throw new Error('Invalid project ID');
  }

  return await Extension.find({ status: 'pending' })
    .populate({
      path: 'projectTeamMember',
      match: { project: projectId },
      populate: [
        {
          path: 'employee',
          select: 'name role employeeId designation'
        },
        {
          path: 'project',
          select: 'name'
        }
      ]
    })
    .then(extensions => extensions.filter(ext => ext.projectTeamMember !== null));
};

module.exports = {
  createExtensionRequest,
  findAllPendingExtensions,
  updateExtensionStatus,
  employeeService,
  getExtensionHistoryByProjectId,
  getProjectTeamMemberById,
  getPendingExtensionsByProjectId
};