const mongoose = require('mongoose');
const projectRepository = require('../project.repository');
const Employee = require('../../../../database/models/employeeSchema');
const Role = require('../../../../database/models/roleSchema');
const Project = require('../../../../database/models/project/projectSchema');
const TimeEntry = require('../../../../database/models/timeEntrySchema');
const ProjectTeamMember = require('../../../../database/models/project/projectTeamMemberSchema');

const getAllProjectByStatus = async (search, status, page, limit) => {
  return await projectRepository.findAllWithSearchStatusAndPagination(search, status, page, limit);
};

const getAllProject = async (search, page = 1,
  limit = 25, status,fetchAll=false) => {
  return await projectRepository.findAllWithSearchAndPagination(search, page, limit, status,fetchAll);
};

async function getRoleIdByName(role) {
  try {
    const roleid = await Role.findOne({ roleName: role });
    if (!roleid) {
      throw new Error(`Role with name "${role}" not found`);
    }
    return roleid._id;
  } catch (error) {
    console.error(`Error fetching role ID for ${role}:`, error);
    throw error;
  }
}

const getProjectById = async (id) => {
  try {
    const objectId = new mongoose.Types.ObjectId(id);
    const project = await Project.findById(objectId).lean();

    if (!project) {
      return null;
    }

    const workedHours = await TimeEntry.aggregate([
      {
        $match: {
          projectId: objectId,
          approved: true
        }
      },
      {
        $group: {
          _id: null,
          totalHours: { $sum: '$duration' }
        }
      }
    ]);

    project.workedhours = workedHours.length > 0 ? workedHours[0].totalHours : 0;

    return project;
  } catch (error) {
    console.error('Error in getProjectById:', error);
    throw error;
  }
};

const addProject = async (projectData) => {
  const existingProject = await projectRepository.findByName(projectData.name);
  if (existingProject) {
    throw new Error('A project with this name already exists');
  }
  
  projectData.initialStartDate = projectData.from;

  delete projectData.tam;
  
  return await projectRepository.create(projectData);
};

const editProjectById = async (id, projectData) => {
  try {
    const currentProject = await projectRepository.findById(id);
    if (!currentProject) {
      throw new Error('Project not found');
    }

    if (projectData.tam === '') {
      projectData.tam = null;
    }

    // Handle TAM changes or removal
    if (currentProject.tam && (projectData.tam !== currentProject.tam.toString() || projectData.tam === null)) {
      const currentTAM = await Employee.findById(currentProject.tam);
      if (currentTAM) {
        currentTAM.TAMofProjects = currentTAM.TAMofProjects.filter(projId => projId.toString() !== id);
        await currentTAM.save();
      }
    }

    // If a new TAM is assigned
    if (projectData.tam) {
      const newTAM = await Employee.findById(projectData.tam);
      if (!newTAM) {
        throw new Error('New TAM employee not found');
      }
      if (!newTAM.TAMofProjects.includes(id)) {
        newTAM.TAMofProjects.push(id);
        await newTAM.save();
      }
    }

    projectData.initialStartDate = currentProject.initialStartDate;
    const updatedProject = await projectRepository.updateById(id, projectData);
    return updatedProject;
  } catch (error) {
    console.error(error);
    throw new Error('Error updating project');
  }
};


//   try {
//     const currentProject = await projectRepository.findById(id);
//     if (!currentProject) {
//       throw new Error('Project not found');
//     }

//     if (projectData.tam === '') {
//       projectData.tam = null;
//     }

//     if (projectData.tam && currentProject.tam?.toString() !== projectData.tam) {
//       const newTAM = await Employee.findById(projectData.tam).populate('role');

//       if (!newTAM) {
//         throw new Error('New TAM employee not found');
//       }

//       if (currentProject.tam) {
//         const currentTAM = await Employee.findById(currentProject.tam).populate('role');
//         if (currentTAM) {
//           currentTAM.TAMofProjects = currentTAM.TAMofProjects.filter(projId => projId.toString() !== id);
          
//           // Check if the current TAM is not a SuperAdmin or ResourceManager before changing the role
//           if (currentTAM.role.roleName !== 'SuperAdmin' && currentTAM.role.roleName !== 'ResourceManager') {
//             currentTAM.role = currentTAM.TAMofProjects.length ? await getRoleIdByName('TAM') : await getRoleIdByName('Employee');
//           }
//           await currentTAM.save();
//         }
//       }

//       newTAM.TAMofProjects.push(id); 
//       await newTAM.save();
    
//     }
//     if (projectData.tam === null) {
//       delete projectData.tam;
//     }

//     projectData.initialStartDate = currentProject.initialStartDate;
//     const updatedProject = await projectRepository.updateById(id, projectData);
//     return updatedProject;
//   } catch (error) {
//     console.error(error);
//     throw new Error('Error updating project');
//   }
// };

// Added missing closing brace here

// const closeProject = async (projectId, employeeId, closureData) => {
//   try {
//     // Find the project by ID
//     const project = await Project.findById(projectId);
//     if (!project) {
//       throw new Error('Project not found');
//     }

//     if (project.closuredate) {
//       throw new Error('Project is already closed');
//     }

//     // Find the employee by employeeId
//     const employee = await Employee.findById(employeeId);
//     if (!employee) {
//       throw new Error('Employee not found');
//     }

//     // Check if the projectId is in the employee's TAMofProjects array
//     const projectIndex = employee.TAMofProjects.indexOf(projectId);
//     if (projectIndex !== -1) {
//       // Remove the project ID from TAMofProjects array
//       employee.TAMofProjects.splice(projectIndex, 1);

//       // Update the employee's role if they have no more TAMofProjects
//       if (employee.TAMofProjects.length === 0) {
//         const employeeRole = await Role.findOne({ roleName: 'Employee' });
//         employee.role = employeeRole._id;
//       }

//       // Save the employee
//       await employee.save();
//     }

//     // Update and close the project
//     const updatedProject = await projectRepository.closeProjectById(projectId, closureData);

//     return updatedProject;
//   } catch (error) {
//     console.error('Error in closeProject:', error);
//     throw error;
//   }
// };

const closeOrReopenProject = async (projectId, employeeId, remarks, closuredate) => {
  try {
    const project = await projectRepository.closeOrReopenProject(
      projectId, 
      employeeId, 
      remarks,
      closuredate  // Pass closure date to repository
    );

    const employee = await Employee.findById(employeeId);
    if (!employee) {
      throw new Error('Employee not found');
    }

    await employee.save();

    return project;
  } catch (error) {
    console.error('Error in closeOrReopenProject:', error);
    throw error;
  }
};




const deleteProjectById = async (id) => {
  try {
    const project = await Project.findById(id);
    if (!project) {
      throw new Error('Project not found');
    }

    const tamId = project.tam;

    // 1. Delete the object from the Projects array in the employee schema
    await Employee.updateMany(
      { 'Projects.projectId': { $in: await ProjectTeamMember.find({ project: id }).distinct('_id') } },
      { $pull: { Projects: { projectId: { $in: await ProjectTeamMember.find({ project: id }).distinct('_id') } } } }
    );

    // 2. Delete associated project team members
    await ProjectTeamMember.deleteMany({ project: id });

    // 3. Hard delete the project
    const deletedProject = await Project.findByIdAndDelete(id);
    if (!deletedProject) {
      throw new Error('Project could not be deleted');
    }

    // Update the TAM's TAMofProjects array
    if (tamId) {
      const tamEmployee = await Employee.findById(tamId);
      if (tamEmployee) {
        // Remove the project ID from TAMofProjects array
        tamEmployee.TAMofProjects = tamEmployee.TAMofProjects.filter(
          projId => projId.toString() !== id.toString()
        );

        // Update the role based on remaining TAMofProjects
        tamEmployee.role = tamEmployee.TAMofProjects.length 
          ? await getRoleIdByName('TAM') 
          : await getRoleIdByName('Employee');

        await tamEmployee.save();
        console.log(`Updated TAM (${tamId}): Removed project ${id} from TAMofProjects`);
      } else {
        console.log(`TAM (${tamId}) not found for project ${id}`);
      }
    } else {
      console.log(`No TAM associated with project ${id}`);
    }

    return deletedProject;
  } catch (error) {
    console.error('Error in deleteProjectById:', error);
    throw error;
  }
};

module.exports = {
  getAllProjectByStatus,
  getProjectById,
  addProject,
  editProjectById,
  deleteProjectById,
  getAllProject,
  // closeProject
  closeOrReopenProject
};