const Project = require('../../../database/models/project/projectSchema');
const Employee = require('../../../database/models/employeeSchema');
const ProjectTeamMember = require('../../../database/models/project/projectTeamMemberSchema');
const applySearchAndPagination = require('../../../utils/paginationAndSearch');
// const {getEmployeeDetails} = require('../../RolesAndAccess/Services/role.service');

const findAllWithSearchStatusAndPagination = async (search, status, page, limit, fetchAll = false) => {
  const searchFields = ['name', 'type'];
  let baseQuery = { visibility: true }; // Add visibility filter

  // Apply status filtering
  if (status === '0') {
    baseQuery.$or = [
      { closuredate: { $exists: false } },
      { closuredate: null }
    ];
  } else if (status === '1') {
    baseQuery.closuredate = { $exists: true, $ne: null };
  } else {
    throw new Error('Invalid status');
  }

  return await applySearchAndPagination(
    Project,
    searchFields,
    search,
    page,
    limit,
    undefined, // populateFields
    fetchAll,  
    baseQuery
  );
};

const findAll = async () => {
  return await Project.find({ visibility: true });
};

const findById = async (id) => {
  return await Project.findOne({ _id: id });
};

const findByName = async (name) => {
  return await Project.findOne({ name: { $regex: new RegExp(name, 'i') }, visibility: true });
};



const findAllWithSearchAndPagination = async (search, page, limit, status, fetchAll = false) => {
  const baseQuery = { visibility: true };

  // Add search criteria if provided
  const searchCriteria = [];
  if (search) {
    searchCriteria.push(
      { name: { $regex: search, $options: 'i' } },
      { type: { $regex: search, $options: 'i' } }
    );
  }

  // Adjust baseQuery based on status
  if (status !== undefined && status !== '') {
    if (status === '0') {
      baseQuery.$or = [
        { closuredate: { $exists: false } },
        { closuredate: { $eq: null } },
        { closuredate: { $eq: '' } }
      ];
    } else if (status === '1') {
      baseQuery.closuredate = { $exists: true, $ne: null };
    } else {
      console.warn('Invalid status:', status);
    }
  }

  if (searchCriteria.length > 0) {
    if (!baseQuery.$and) {
      baseQuery.$and = [];
    }
    baseQuery.$and.push({ $or: searchCriteria });
  }

  // Base aggregation pipeline without pagination
  const basePipeline = [
    {
      $match: baseQuery,
    },
    {
      $lookup: {
        from: 'timeentries',
        localField: '_id',
        foreignField: 'projectId',
        as: 'timeEntries',
      },
    },
    {
      $unwind: {
        path: '$timeEntries',
        preserveNullAndEmptyArrays: true,
      },
    },
    {
      $group: {
        _id: '$_id',
        name: { $first: '$name' },
        type: { $first: '$type' },
        from: { $first: '$from' },
        end: { $first: '$end' },
        estimatedhours: { $first: '$estimatedhours' },
        tam: { $first: '$tam' },
        description: { $first: '$description' },
        milestone: { $first: '$milestone' },
        skillsets: { $first: '$skillsets' },
        visibility: { $first: '$visibility' },
        closuredate: { $first: '$closuredate' },
        remarks: { $first: '$remarks' },
        workedhours: {
          $sum: {
            $cond: [
              { $eq: ['$timeEntries.approved', true] },
              '$timeEntries.duration',
              0
            ]
          }
        },
      },
    },
    {
      $project: {
        name: { $ifNull: ['$name', ''] },
        workedhours: { $ifNull: ['$workedhours', 0] },
        type: { $ifNull: ['$type', ''] },
        from: { $ifNull: ['$from', ''] },
        end: { $ifNull: ['$end', ''] },
        closuredate: { $ifNull: ['$closuredate', ''] },
        estimatedhours: { $ifNull: ['$estimatedhours', ''] }
      },
    },
    {
      $sort: { _id: 1 },
    }
  ];

  try {
    if (fetchAll) {
      // If fetchAll is true, return all results without pagination
      const allProjects = await Project.aggregate([
        ...basePipeline,
        {
          $group: {
            _id: null,
            results: { $push: '$$ROOT' },
            total: { $sum: 1 }
          }
        },
        {
          $project: {
            _id: 0,
            results: 1,
            total: 1,
            page: { $literal: 1 },
            pageSize: { $literal: -1 }
          }
        }
      ]);
      return allProjects.length > 0 ? allProjects[0] : { results: [], total: 0, page: 1, pageSize: -1 };
    } else {
      // If fetchAll is false, apply pagination
      const skip = (page - 1) * limit;
      const paginatedProjects = await Project.aggregate([
        ...basePipeline,
        {
          $facet: {
            results: [{ $skip: skip }, { $limit: limit }],
            total: [{ $count: 'count' }],
          },
        },
        {
          $unwind: '$total',
        },
        {
          $project: {
            results: 1,
            total: '$total.count',
            page: { $literal: page },
            pageSize: { $literal: limit },
          },
        }
      ]);
      return paginatedProjects.length > 0 ? paginatedProjects[0] : { results: [], total: 0, page, pageSize: limit };
    }
  } catch (error) {
    console.error('Error in findAllWithSearchAndPagination:', error);
    throw new Error('Database query failed');
  }
};




const create = async (projectData) => {
  const project = new Project(projectData);
  return await project.save();
};

const updateById = async (id, projectData) => {
  return await Project.findOneAndUpdate({ _id: id, visibility: true }, projectData, { new: true });
};

const deleteById = async (id) => {
  return await Project.findByIdAndDelete(id);
};

// const closeProjectById = async (id, closureData) => {
//   return await Project.findOneAndUpdate(
//     { 
//       _id: id, 
//       visibility: true, 
//       closuredate: null  
//     },
//     { 
//       closuredate: new Date(),
//       remarks: closureData.remarks,
//       isreopen: false 
//     },
//     { new: true }
//   );
// };

const closeOrReopenProject = async (projectId, employeeId, remarks, closuredate) => {
  const project = await Project.findById(projectId);

  if (!project) {
    throw new Error('Project not found');
  }

  const isReopening = !!project.closuredate;

  if (isReopening) {
    // Reopening logic
    project.closuredate = null;
    project.isreopen = true;
    project.remarks = remarks || project.remarks;

    await Employee.findByIdAndUpdate(
      employeeId,
      { $addToSet: { TAMofProjects: projectId } }
    );
  } else {
    // Closing logic
    project.closuredate = closuredate ? new Date(closuredate) : new Date();
    project.isreopen = false;
    project.remarks = remarks;

    await Employee.findByIdAndUpdate(
      employeeId,
      { $pull: { TAMofProjects: projectId } }
    );
  }

  // Update employee statuses
  // Only fetch team members where visibility is true
  const projectTeamMembers = await ProjectTeamMember.find({ 
    project: projectId,
    visibility: true
  });

  for (const teamMember of projectTeamMembers) {
    if (isReopening) {
      // When reopening, update only if the project has 'On bench' status
      await Employee.updateOne(
        { 
          _id: teamMember.employee,
          'Projects': {
            $elemMatch: {
              'projectId': teamMember._id,
              'benchStatus': 'On bench'
            }
          }
        },
        { 
          $set: { 
            'Projects.$.benchStatus': 'active',
            'Projects.$.active': true
          }
        }
      );
    } else {
      // When closing, update only if the project has 'active' status
      await Employee.updateOne(
        { 
          _id: teamMember.employee,
          'Projects': {
            $elemMatch: {
              'projectId': teamMember._id,
              'benchStatus': 'active'
            }
          }
        },
        { 
          $set: { 
            'Projects.$.benchStatus': 'On bench',
            'Projects.$.active': false
          }
        }
      );
    }
  }

  await project.save();
  return project;
};

module.exports = {
  findById,
  create,
  findByName,
  updateById,
  deleteById,
  findAll,
  findAllWithSearchAndPagination,
  findAllWithSearchStatusAndPagination,
  // closeProjectById,
  closeOrReopenProject
};