const ProjectTeamMember = require('../../../database/models/project/projectTeamMemberSchema');
const applySearchAndPagination = require('../../../utils/paginationAndSearch');

const findAll = async () => {
  return await ProjectTeamMember.find({ visibility: true }).populate([
    {
      path: 'employee',
      select: 'name type employeeId designation skillset role',
      populate: [
        {
          path: 'skillset',
          select: 'name'
        },
        {
          path: 'role',
          select: 'roleName'
        }
      ]
    },
    {
      path: 'project',
      select: 'name type startDate endDate estimatedHours workedHours tam description closuredate'
    }
  ]);
};


const findById = async (id) => {
  return await ProjectTeamMember.findById({ _id: id, visibility: true }).populate({
    path: 'employee',
    select: 'name type employeeId designation skillset Projects',
    populate: [
      {
        path: 'skillset',
        select: 'name'
      },
      {
        path: 'Projects.projectId',  // Populating Projects field from Employee schema
        select: 'benchStatus active',  // Fields from ProjectTeamMember schema
        populate: {
          path: 'project',  // Populate project from ProjectTeamMember schema
          select: 'name type from end estimatedhours workedhours tam description',  // Fields from Project schema
          populate: {
            path: 'tam',  // Populate TAM field from Project schema
            select: 'name employeeId designation'  // Fields from Employee schema
          }
        }
      }
    ]
  });
};


const findByProjectId = async (projectId, search, page, limit, fetchAll = false) => {
  // Adjusted search fields to correctly reference nested employee fields
  const searchFields = ['role', 'employee.name', 'employee.employeeId', 'employee.designation'];

  const populateFields = [
    {
      path: 'employee',
      select: 'name type employeeId designation skillset Projects',
      populate: [
        {
          path: 'skillset',
          select: 'name'
        },
        {
          path: 'Projects.projectId',  // Populating Projects field from Employee schema
          select: 'benchStatus active',  // Fields from ProjectTeamMember schema
          populate: {
            path: 'project',  // Populate project from ProjectTeamMember schema
            select: 'name type from end estimatedhours workedhours tam description',  // Fields from Project schema
            populate: {
              path: 'tam',  // Populate TAM field from Project schema
              select: 'name employeeId designation'  // Fields from Employee schema
            }
          }
        }
      ]
    }
  ];

  const baseQuery = { project: projectId, visibility: true };

  return await applySearchAndPagination(
    ProjectTeamMember,
    searchFields,
    search,
    page,
    limit,
    populateFields,
    fetchAll,  // Pass the fetchAll parameter to control pagination
    baseQuery
  );
};



const createMany = async (teamMemberData) => {
  const createdMembers = await ProjectTeamMember.create(teamMemberData);
  return Array.isArray(createdMembers) ? createdMembers : [createdMembers];
};

const updateById = async (id, teamMemberData) => {
  const updatedMember = await ProjectTeamMember.findOneAndUpdate(
    { _id: id, visibility: true },
    teamMemberData,
    { new: true }
  ).populate({
    path: 'employee',
    select: 'name type employeeId designation skillset',
    populate: {
      path: 'skillset',
      select: 'name'
    }
  });

  if (!updatedMember) {
    throw new Error('Team member not found');
  }

  return updatedMember;
};

const deleteById = async (id) => {
  return await ProjectTeamMember.findByIdAndUpdate(id, { visibility: false }, { new: true });
};

module.exports = {
  findAll,
  findById,
  findByProjectId,
  createMany,
  updateById,
  deleteById
};
