const Project = require('../../../database/models/project/projectSchema');
const addSkillsetsToProject = async (projectId, skillsetIds) => {
  try {
    const project = await Project.findById(projectId);
    if (!project) {
      return { status: 404, message: 'Project not found' };
    }

    // Avoid adding duplicate skillsets
    skillsetIds.forEach(skillsetId => {
      if (!project.skillsets.includes(skillsetId)) {
        project.skillsets.push(skillsetId);
      }
    });

    await project.save();
    return { status: 200, project };
  } catch (error) {
    return { status: 500, message: 'Internal server error', error };
  }
};

const getProjectWithSkillsets = async (projectId, search, page, limit) => {
  const skillsetMatch = {
    $or: [
      { name: { $regex: search, $options: 'i' } },
      // { category: { $regex: search, $options: 'i' } } // Add this line if you want category to be searchable
    ]
  };

  // Fetch the project with populated skillsets matching the search criteria
  const project = await Project.findById(projectId).populate({
    path: 'skillsets',
    match: skillsetMatch,
    options: {
      skip: (page - 1) * limit,
      limit: parseInt(limit, 10),
      sort: { name: 1 } 
    },
    select: 'name category' // Select only necessary fields
  });

  if (!project) {
    return { project: null, totalSkillsets: 0 }; // Return if project is not found
  }

  // Get the total count of skillsets that match the search criteria
  const totalSkillsets = await Project.aggregate([
    { $match: { _id: project._id } },
    { $unwind: '$skillsets' },
    {
      $lookup: {
        from: 'skillsets',
        localField: 'skillsets',
        foreignField: '_id',
        as: 'skillsetsDetails'
      }
    },
    { $unwind: '$skillsetsDetails' },
    { $match: skillsetMatch },
    { $sort: { 'skillsetsDetails.name': 1 } },
    { $count: 'totalSkillsets' }
  ]);

  return {
    project,
    totalSkillsets: totalSkillsets[0] ? totalSkillsets[0].totalSkillsets : 0 // Handle no results case
  };
};





const deleteSkillsetFromProject = async (projectId, skillsetId) => {
  try {
    const project = await Project.findById(projectId);
    if (!project) {
      return { message: 'Project not found' };
    }

    // Filter out the skillset to delete from the array
    project.skillsets = project.skillsets.filter(skillset => skillset.toString() !== skillsetId);
    await project.save();

    return project;
  } catch (error) {
    console.error('Error in repository deleting skillset from project:', error);
    throw error;
  }
};




module.exports = {
  addSkillsetsToProject,
  getProjectWithSkillsets,
  deleteSkillsetFromProject
};