const resourceRepository = require('../requsestResource.repository');


exports.getAllPendingRequests = async (search, page, limit, fetchAll = false) => {
  return await resourceRepository.findAllPendingRequests(search, page, limit, fetchAll);
};



exports.allocateResources = async (allocationData) => {
  if (allocationData.numberOfEmployees <= 0) {
    throw new Error('Invalid number of employees');
  }
  return await resourceRepository.saveAllocation(allocationData);
};

exports.updateRequestStatus = async (id, updateData) => {
  const { status, updatedEmployeeRole, allocatedEmployees, requestedEmployeeNames } = updateData;
  
  const currentRequest = await resourceRepository.findAllocationById(id);
  
  if (!currentRequest) {
    throw new Error('Allocation request not found');
  }
  
  // Convert status to lowercase for case-insensitive comparison
  const lowerStatus = status.toLowerCase();

  if (lowerStatus === 'rejected') {
    currentRequest.status = 'Rejected';
    // Add a rejection entry to the history
    currentRequest.history.push({
      updatedEmployeeRole: updatedEmployeeRole || 'System',
      status: 'rejected',
      date: new Date(),
      requestedEmployeeNames: []
    });
  } else if (lowerStatus === 'accepted') {
    currentRequest.allocatedEmployees = allocatedEmployees;
    
    currentRequest.history.push({
      requestedEmployeeNames,
      updatedEmployeeRole,
      status: 'accepted',
      date: new Date()
    });
    
    if (currentRequest.allocatedEmployees === currentRequest.numberOfEmployees) {
      currentRequest.status = 'Completed';
    } else {
      currentRequest.status = 'Pending';
    }
  } else {
    throw new Error(`Invalid status update: '${status}'. Expected 'Accepted' or 'Rejected'.`);
  }
  
  const updatedRequest = await currentRequest.save();
  
  return updatedRequest;
};

exports.getAllHistoryByProjectId = async (projectId) => {
  const allocations = await resourceRepository.findAllocationsByProjectId(projectId);

  const history = {
    all: [] // Create a single array to hold all history entries
  };

  if (allocations.length === 0) {
    return history;
  }



  allocations.forEach(allocation => {
    const { requestedBy, numberOfEmployees, skillSet, createdAt, status, updatedAt } = allocation;

    // Add the initial request to the all history
    history.all.push({
      message: `${requestedBy} has requested allocation of ${numberOfEmployees} additional resources for ${skillSet ? skillSet.name : 'Unknown Skill'}`,
      status: 'pending',
      date: new Date(createdAt) // Store date as a Date object for sorting
    });

    if (status === 'Rejected') {
      const rejectionRole = allocation.history.length > 0 
        ? allocation.history[allocation.history.length - 1].updatedEmployeeRole 
        : requestedBy;  // Use the last history item's updatedEmployeeRole or fall back to requestedBy if history is empty
      
      history.all.push({
        message: `Request for allocation of ${numberOfEmployees} additional resources for ${skillSet ? skillSet.name : 'Unknown Skill'} has been rejected by ${rejectionRole}`,
        status: 'rejected',
        date: new Date(updatedAt) // Store date as a Date object for sorting
      });
    }
    
    if (allocation.history && Array.isArray(allocation.history)) {
      allocation.history.forEach(historyItem => {
        if (historyItem.status === 'accepted') {
          history.all.push({
            message: `${historyItem.updatedEmployeeRole} has allocated ${historyItem.requestedEmployeeNames.join(', ')} for ${skillSet ? skillSet.name : 'Unknown Skill'}`,
            status: 'accepted',
            date: new Date(historyItem.date) // Store date as a Date object for sorting
          });
        }
      });
    }
  });

  // Sort the history.all array by date
  history.all.sort((a, b) => new Date(a.date) - new Date(b.date));

  return history;
};

exports.getPendingRequests = async (projectId) => {
  return await resourceRepository.findPendingRequests(projectId);
};