const cron = require('node-cron');
const Employee = require('../database/models/employeeSchema');
const Timesheet = require('../database/models/timesheetSchema');

function formatDate(date) {
  const days = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
  const months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
  
  const dayName = days[date.getDay()];
  const monthName = months[date.getMonth()];
  const day = String(date.getDate()).padStart(2, '0');
  
  return `${dayName},${monthName} ${day}`;
}

async function addEmployeeMessage(employeeId, messageText, date) {
  try {
    const employee = await Employee.findById(employeeId);
    if (employee) {
      employee.messages.push({
        message: messageText,
        date: date,
        status: 'Active',
      });

      employee.messages.sort((a, b) => b.date - a.date);

      await employee.save();
      console.log(`Message added to employee ${employeeId} and messages sorted`);
    } else {
      console.log(`Employee ${employeeId} not found`);
    }
  } catch (error) {
    console.error('Error adding message to employee:', error);
  }
}

function isEmployeeEligible(dateOfJoining) {
  const [day, month, year] = dateOfJoining.split('-').map(Number);
  const joinDate = new Date(year, month - 1, day);

  const currentDate = new Date();
  const diffTime = Math.abs(currentDate - joinDate);
  const diffDays = Math.ceil(diffTime / (1000 * 60 * 60 * 24));
  return diffDays > 1;
}

async function checkTimesheets(checkDate) {
  try {
    const employees = await Employee.find({ visibility: true });

    for (const employee of employees) {
      if (!isEmployeeEligible(employee.dateOfJoining)) {
        console.log(`Skipping timesheet check for ${employee.name} (joined recently)`);
        continue;
      }

      const employeeId = employee._id;

      checkDate.setUTCHours(0, 0, 0, 0); // Set time to start of day in UTC
      const nextDay = new Date(checkDate);
      nextDay.setUTCDate(nextDay.getUTCDate() + 1);

      const timesheet = await Timesheet.findOne({
        employeeId: employeeId,
        date: {
          $gte: checkDate,
          $lt: nextDay
        }
      });

      const formattedDate = formatDate(checkDate);

      if (!timesheet || (timesheet && timesheet.entryIds.length === 0 && !timesheet.submitted)) {
        await addEmployeeMessage(
          employeeId,
          `Your timesheet for ${formattedDate} has no entries. Please add your time entries and submit it by clicking here.`,
          new Date(checkDate)
        );
      } else if (timesheet.entryIds.length > 0 && !timesheet.submitted) {
        await addEmployeeMessage(
          employeeId,
          
          `Your timesheet for ${formattedDate} hasn't been submitted yet. Kindly click here to complete your submission.`,

          new Date(checkDate)
        );
      }
    }
  } catch (error) {
    console.error('Error checking timesheets:', error);
  }
}

function startCronJobs() {
  // Daily check at 23:59 PM (Monday to Friday)
  cron.schedule('59 23 * * 1-5', () => {
    console.log('Running daily timesheet check...');
    const today = new Date();
    checkTimesheets(today);
  });

  // Weekly check on Saturday at 6:00 AM for the past five days
  // cron.schedule('0 6 * * 6', () => {
  //   console.log('Running weekly timesheet check for the past five days...');
  //   const today = new Date();
  //   for (let i = 1; i <= 5; i++) {
  //     const checkDate = new Date(today);
  //     checkDate.setDate(today.getDate() - i);
  //     if (checkDate.getDay() !== 0 && checkDate.getDay() !== 6) { // Skip weekends
  //       checkTimesheets(checkDate);
  //     }
  //   }
  // });

  console.log('Cron jobs started. Timesheets will be checked daily at 23:59 PM (Monday to Friday) and weekly on Saturday at 6:00 AM for the past five days.');
}

module.exports = startCronJobs;